import { type Signal, type ValueEqualityFn } from "@angular/core";
import { type Observable, Subject } from "rxjs";
import { DuplicateError, type ObjectsComparator, type ObjectsComparatorFn } from "./comparator";
import type { CollectionInterface, CollectionOptions, CreateManyParams, CreateParams, DeleteManyParams, DeleteParams, DuplicatesMap, FetchedItems, ReadManyParams, ReadOneParams, ReadParams, RefreshManyParams, RefreshParams, UpdateManyParams, UpdateParams } from "./types";
export declare class Collection<T, UniqueStatus = unknown, Status = unknown> implements CollectionInterface<T, UniqueStatus, Status> {
    protected comparator: ObjectsComparator;
    protected throwOnDuplicates?: string | false;
    protected allowFetchedDuplicates: boolean;
    protected onDuplicateErrCallbackParam: DuplicateError;
    protected errReporter?: (...args: any[]) => any;
    protected readonly onCreate: Subject<T[]>;
    protected readonly onRead: Subject<T[]>;
    protected readonly onUpdate: Subject<T[]>;
    protected readonly onDelete: Subject<Partial<T>[]>;
    protected isFirstItemsRequest: boolean;
    protected onFirstItemsRequest?: Function;
    protected readonly equalItems: ValueEqualityFn<T[]>;
    protected readonly state: {
        readonly $items: import("@angular/core").WritableSignal<T[]>;
        readonly $totalCountFetched: import("@angular/core").WritableSignal<number | undefined>;
        readonly $isCreating: import("@angular/core").WritableSignal<boolean>;
        readonly $isReading: import("@angular/core").WritableSignal<boolean>;
        readonly $isBeforeFirstRead: import("@angular/core").WritableSignal<boolean>;
        readonly $updatingItems: import("@angular/core").WritableSignal<T[]>;
        readonly $deletingItems: import("@angular/core").WritableSignal<T[]>;
        readonly $refreshingItems: import("@angular/core").WritableSignal<T[]>;
        readonly $status: import("@angular/core").WritableSignal<Map<UniqueStatus, T>>;
        readonly $statuses: import("@angular/core").WritableSignal<Map<T, Set<Status>>>;
    };
    readonly $items: Signal<T[]>;
    readonly $totalCountFetched: Signal<number | undefined>;
    readonly $isCreating: Signal<boolean>;
    readonly $isReading: Signal<boolean>;
    readonly $isBeforeFirstRead: Signal<boolean>;
    readonly $updatingItems: Signal<T[]>;
    readonly $deletingItems: Signal<T[]>;
    readonly $refreshingItems: Signal<T[]>;
    readonly $status: Signal<Map<UniqueStatus, T>>;
    readonly $statuses: Signal<Map<T, Set<Status>>>;
    readonly $isUpdating: Signal<boolean>;
    readonly $isDeleting: Signal<boolean>;
    readonly $isMutating: Signal<boolean>;
    readonly $isSaving: Signal<boolean>;
    readonly $isProcessing: Signal<boolean>;
    readonly $mutatingItems: Signal<T[]>;
    readonly $processingItems: Signal<T[]>;
    constructor(options?: CollectionOptions);
    protected init(): void;
    protected asyncInit(): void;
    create(params: CreateParams<T>): Observable<T>;
    createMany(params: CreateManyParams<T>): Observable<T[] | FetchedItems<T>>;
    read(params: ReadParams<T>): Observable<T[] | FetchedItems<T>>;
    readOne(params: ReadOneParams<T>): Observable<T>;
    readMany(params: ReadManyParams<T>): Observable<T[] | FetchedItems<T>>;
    refresh(params: RefreshParams<T>): Observable<T>;
    refreshMany(params: RefreshManyParams<T>): Observable<T[] | FetchedItems<T>>;
    update(params: UpdateParams<T>): Observable<T>;
    updateMany(params: UpdateManyParams<T>): Observable<T[] | FetchedItems<T>>;
    delete<R = unknown>(params: DeleteParams<T, R>): Observable<R>;
    deleteMany<R = unknown>(params: DeleteManyParams<T, R>): Observable<R[]>;
    setUniqueStatus(status: UniqueStatus, item: T, active?: boolean): void;
    deleteUniqueStatus(status: UniqueStatus): void;
    setItemStatus(item: T, status: Status): void;
    deleteItemStatus(item: T, status: Status): void;
    isItemDeleting(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemRefreshing(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemUpdating(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemMutating(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    isItemProcessing(itemSource: Partial<T> | Signal<Partial<T> | undefined>): Signal<boolean>;
    getItem(filter: Partial<T> | undefined | Signal<Partial<T> | undefined>, equalFn?: ValueEqualityFn<T | undefined> | undefined): Signal<T | undefined>;
    getItemByField<K extends keyof T>(field: K | K[], fieldValue: T[K] | Signal<T[K] | undefined>, equalFn?: ValueEqualityFn<T | undefined> | undefined): Signal<T | undefined>;
    setThrowOnDuplicates(message: string | undefined): void;
    setAllowFetchedDuplicates(allowFetchedDuplicates: boolean): void;
    setOnDuplicateErrCallbackParam(onDuplicateErrCallbackParam: any): void;
    hasItemIn(item: Partial<T>, arr: Partial<T>[]): boolean;
    getTrackByFieldFn(field: string): (i: number, item: any) => any;
    listenForItemsUpdate(items: Partial<T>[]): Observable<T[]>;
    listenForItemsDeletion(items: Partial<T>[]): Observable<Partial<T>[]>;
    listenForCreate(): Observable<T[]>;
    listenForRead(): Observable<T[]>;
    listenForUpdate(): Observable<T[]>;
    listenForDelete(): Observable<Partial<T>[]>;
    setComparator(comparator: ObjectsComparator | ObjectsComparatorFn): void;
    setOptions(options?: CollectionOptions | null): void;
    uniqueItems(items: T[]): T[];
    getItemByPartial(partItem: Partial<T>, items?: T[]): T | undefined;
    getDuplicates(items: T[]): DuplicatesMap<T> | null;
    setAfterFirstReadHandler(cb: Function | undefined): void;
    setOnFirstItemsRequest(cb: Function | undefined): void;
    protected getWith(items: T[], item: Partial<T> | Partial<T>[]): T[];
    protected getWithout(items: T[], item: Partial<T> | Partial<T>[]): T[];
    protected findInSet<Ret extends Partial<T>>(seek: Partial<T>[], items: Ret[]): Ret[];
    protected upsertOne(item: T, items: T[]): T[] | 'duplicate';
    protected hasDuplicates(items: T[]): T | null;
    protected upsertMany(items: T[], toItems: T[]): T[] | {
        duplicate: T;
        preExisting?: boolean;
    };
    protected duplicateNotAdded(item: T, items: T[]): void;
    protected callCb(cb: ((_: any) => any) | undefined, param: any): void;
    protected callErrReporter(...args: any): void;
    protected reportRuntimeError(...args: any): void;
    protected getDecrementedTotalCount<R>(response: R[], itemsCount: number, prevTotalCount?: number, decrementTotalCount?: boolean | number | string): number | null;
    protected toObservableFirstValue<Src>(s: Observable<Src> | Signal<Src>): Observable<Src>;
    protected forkJoinSafe<Src>(sources: Observable<Src>[] | Signal<Src>[] | (Observable<Src> | Signal<Src>)[]): Observable<Src[]>;
}
